<%-- $Id: settings.jsp 4511 2008-09-11 20:04:27Z jari $
	------------------------------------------------------------------
	Copyright (C) 2005 Nicklas Nordborg
	Copyright (C) 2006 Jari Hakkinen, Nicklas Nordborg
	Copyright (C) 2007 Martin Svensson

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
	------------------------------------------------------------------

	This page is used to display and modify the attributes of
	the logged in user. It displays a tabbed dialogue:

	@param page
		The name active page of the tabbed dialog.
		- contact: Contact information for the user, address, phone, email, etc.
		- password: The password for the user
		- other: Other information

	Saving the form invokes the submit_user.jsp page with cmd=SaveSettings

	@author Nicklas
	@version 2.0
--%>
<%@ page session="false"
	import="net.sf.basedb.core.*"
	import="net.sf.basedb.clients.web.*"
	import="net.sf.basedb.clients.web.util.HTML"
	import="net.sf.basedb.util.formatter.Formatter"	
	import="net.sf.basedb.clients.web.formatter.FormatterFactory"
	import="net.sf.basedb.clients.web.formatter.FormatterSettings"
	import="java.util.Date"
	import="java.util.List"		
%>
<%@ taglib prefix="base" uri="/WEB-INF/base.tld" %>
<%@ taglib prefix="t" uri="/WEB-INF/tab.tld" %>
<%
final SessionControl sc = Base.getExistingSessionControl(pageContext, true);
final String ID = sc.getId();
final String activePage = request.getParameter("page");
final float scale = Base.getScale(sc);
final DbControl dc = sc.newDbControl();
final ItemContext cc = Base.getAndSetCurrentContext(sc, Item.USER, null, null);
try
{
	final User user = User.getById(dc, sc.getLoggedInUserId());
	final boolean writePermission = user.hasPermission(Permission.RESTRICTED_WRITE);
	final String clazz = writePermission ? "class=\"text\"" : "class=\"text disabled\" disabled readonly";
	final String requiredClazz = writePermission ? "class=\"text required\"" : "class=\"text required disabled\" disabled readonly";
	final String passwordClass = writePermission && Application.isUsingInternalAuthentication() ? 
		"text" : "class=\"text disabled\" disabled readonly";
	dc.detachItem(user);
	sc.setSessionSetting("user", user);
	
	List<ExtendedProperty> extendedProperties = ExtendedProperties.getProperties("UserData");
	
	Formatter<Date> dateFormatter = FormatterFactory.getDateFormatter(sc);
	String dateFormat = FormatterSettings.getDateFormat(sc);
	String jsDateFormat = HTML.javaScriptEncode(dateFormat);
	String htmlDateFormat = HTML.encodeTags(dateFormat);
	%>

	<base:page type="popup" title="<%="Information for "+HTML.encodeTags(user.getName())%>">
	<base:head scripts="tabcontrol.js" styles="tabcontrol.css">
		<script language="JavaScript">
		// Validate the "Contact information" tab
		function validateContact()
		{
			return true;
		}
		// Validate the "Password" tab - check that both passwords match
		function validatePassword()
		{
			var frm = document.forms["user"];
			if (frm.password.value != frm.retype_password.value)
			{
				frm.retype_password.focus();
				alert("The two passwords do not match!");
				return false;
			} 
			return true;
		}
		
		// Validate the "Extended properties" tab
		function validateExtendedProperties()
		{
			var frm = document.forms['user'];
			var field;
			<%
			if (extendedProperties != null)
			{
				for (ExtendedProperty ep : extendedProperties)
				{
					Type type = ep.getType();
					String name = ep.getName();
					%>
					field = frm['<%=name%>'];
					<%
					if (type == Type.DATE)
					{
						%>
						if (!Dates.isDate(field.value))
						{
							alert("'"+field.value+"' is not a valid date for '<%=HTML.javaScriptEncode(ep.getTitle())%>'");
							field.focus();
							return false;
						}
						<%
					}
					if (!ep.isNullable())
					{
						if (type == Type.BOOLEAN)
						{
							%>
							if (Forms.getCheckedRadio(field) == null)
							{
								alert("You must select a value for '<%=HTML.javaScriptEncode(ep.getTitle())%>'");
								field.focus();
								return false;
							}
							<%
						}
						else
						{
							%>
							if (Main.trimString(field.value) == '')
							{
								alert("You must enter a value for '<%=HTML.javaScriptEncode(ep.getTitle())%>'.");
								field.focus();
								return false;
							}
							<%
						}
					}
				}
			}
			%>
			return true;
		}
		
		// Submit the form
		function saveSettings()
		{
			var frm = document.forms["user"];
			if (TabControl.validateActiveTab('settings'))
			{
				frm.submit();
			}
		}
		</script>
	</base:head>
	<base:body onload="Forms.fixTextareaBug('user');">
	<p>

	<form action="submit_user.jsp?ID=<%=ID%>" method="post" name="user" onsubmit="return false;">
	<input type=hidden name="cmd" value="SaveSettings">
	
	<h3 class="docked">Information for <%=HTML.encodeTags(user.getName())%> <base:help tabcontrol="settings" /></h3>
	<t:tabcontrol active="<%=activePage%>" id="settings" 
		contentstyle="<%="height: "+(int)(scale*240)+"px;"%>" position="bottom" remember="false">
	
	<t:tab id="contact" title="Contact information" validate="validateContact()" helpid="userpreferences.contact">
		<table class="form" cellspacing=0>
		<tr>
			<td class="prompt">Full name</td>
			<td><%=HTML.encodeTags(user.getName())%></td>
		</tr>
		<tr>
			<td class="prompt">Email</td>
			<td><input <%=clazz%> type="text" name="email" value="<%=HTML.encodeTags(user.getEmail())%>" size="40" maxlength="<%=User.MAX_EMAIL_LENGTH%>"></td>
		</tr>
		<tr>
			<td class="prompt">Organisation</td>
			<td><input <%=clazz%> type="text" name="organisation" value="<%=HTML.encodeTags(user.getOrganisation())%>" size="40" maxlength="<%=User.MAX_ORGANISATION_LENGTH%>"></td>
		</tr>
		<tr valign=top>
			<td class="prompt">Address</td>
			<td>
				<textarea <%=clazz%> rows="4" cols="40" name="address" wrap="virtual"><%=HTML.encodeTags(user.getAddress())%></textarea>
				<a href="javascript:Main.zoom('Address', 'user', 'address', <%=!writePermission%>)" title="Edit in larger window"><base:icon image="zoom.gif" /></a>
			</td>
		</tr>
		<tr>
			<td class="prompt">Phone</td>
			<td><input <%=clazz%> type="text" name="phone" value="<%=HTML.encodeTags(user.getPhone())%>" size="40" maxlength="<%=User.MAX_PHONE_LENGTH%>"></td>
		</tr>
		<tr>
			<td class="prompt">Fax</td>
			<td><input <%=clazz%> type="text" name="fax" value="<%=HTML.encodeTags(user.getFax())%>" size="40" maxlength="<%=User.MAX_FAX_LENGTH%>"></td>
		</tr>
		<tr>
			<td class="prompt">Url</td>
			<td><input <%=clazz%> type="text" name="url" value="<%=HTML.encodeTags(user.getUrl())%>" size="40" maxlength="<%=User.MAX_URL_LENGTH%>"></td>
		</tr>
		</table>
	</t:tab>
	
	<t:tab id="password" title="Password" validate="validatePassword()" helpid="userpreferences.password">
		<% if (!Application.isUsingInternalAuthentication() && writePermission)
		{
			%>
			<div class="error">Base is using external authentication. Password cannot be changed here.</div>
			<%
		}
		%>
		<table class="form" cellspacing=0>
		<tr>
			<td class="prompt">Login</td>
			<td><%=HTML.encodeTags(user.getLogin())%></td>
		</tr>
		<tr>
			<td class="prompt">New password</td>
			<td><input <%=passwordClass%> type="password" name="password" value="" size="30" maxlength="30"></td>
		</tr>
		<tr>
			<td class="prompt">Retype password</td>
			<td><input <%=passwordClass%> type="password" name="retype_password" value="" size="30" maxlength="30"></td>
		</tr>
		</table>
		<div align=right>&nbsp;<i>- leave empty to not change the password</i></div>
	</t:tab>

	<t:tab id="other" title="Other information" 
		validate="validateExtendedProperties()" helpid="userpreferences.other">
		<table class="form" cellspacing="0">
		<%
		if (extendedProperties != null)
		{
			for (ExtendedProperty ep : ExtendedProperties.getProperties("UserData"))
			{
				String name = ep.getName();
				Type type = ep.getType();
				boolean required = !ep.isNullable();
				Object value = user == null ? cc.getPropertyValue(name) : user.getExtended(name);
				String theClazz = required ? requiredClazz : clazz;
				%>
				<tr valign="top">
					<td class="prompt"><%=HTML.encodeTags(ep.getTitle())%></td>
					<td>
					<%
					if (type == Type.INT || type == Type.LONG)
					{
						%>
						<input <%=theClazz%> type="text" name="<%=name%>" 
							value="<%=value == null ? "" : value%>"
							size="20" maxlength="20" onkeypress="return Numbers.integerOnly(event)"
							>
						<%
					}
					else if (type == Type.FLOAT || type == Type.DOUBLE)
					{
						%>
						<input <%=theClazz%> type="text" name="<%=name%>"
							value="<%=value == null ? "" : value%>"
							size="20" maxlength="20" onkeypress="return Numbers.numberOnly(event)"
							>
						<%
					}
					else if (type == Type.STRING)
					{
						%>
						<input <%=theClazz%> type="text" name="<%=name%>"
							value="<%=HTML.encodeTags((String)value)%>"
							size="40" maxlength="<%=ep.getLength()%>"
							>
						<%
					}
					else if (type == Type.TEXT)
					{
						%>
						<textarea <%=theClazz%> name="<%=name%>" rows="6" cols="40"
							><%=HTML.encodeTags((String)value)%></textarea>
						<a href="javascript:Main.zoom(
							'<%=HTML.javaScriptEncode(ep.getTitle())%>', 
							'reporter', '<%=name%>')" 
							title="Edit in larger window"><base:icon image="zoom.gif" /></a>
						<%
					}
					else if (type == Type.BOOLEAN)
					{
						Boolean b = (Boolean)value;
						if (!required)
						{
							%>
							<input type="radio" name="<%=name%>" value=""
								<%=b == null ? "checked" : ""%> 
								><i>- not specified -</i><br>
							<%
						}
						%>
						<input type="radio" name="<%=name%>" value="true" 
							<%=b != null && b == true ? "checked" : ""%>
							>true<br>
						<input type="radio" name="<%=name%>" value="false" 
							<%=b != null && b == false ? "checked" : ""%>
							>false
						<%
					}
					else if (type == Type.DATE)
					{
						%>
						<table>
						<tr>
						<td>
							<input <%=theClazz%> type="text" name="<%=name%>" 
								value="<%=dateFormatter.format((Date)value)%>"
								size="20" maxlength="20" title="Enter date in format: <%=htmlDateFormat%>"
								>
						</td>
						<td>
							<base:button 
								onclick="<%="Dates.selectDate('"+HTML.javaScriptEncode(ep.getTitle())+"', 'reporter', '"+name+"', null, '"+jsDateFormat +"')"%>" 
								image="calendar.png"
								title="Calendar&hellip;" 
								tooltip="Select a date from a calendar" 
							/>
						</td>
						</tr>
						</table>
						<%
					}
					%>
					</td>
				</tr>
				<%
			}
		}
		%>
		<tr valign="top">
			<td class="prompt">Description</td>
			<td>
				<textarea <%=clazz%> rows="5" cols="40" name="description" wrap="virtual"><%=HTML.encodeTags(user.getDescription())%></textarea>
				<a href="javascript:Main.zoom('Description', 'user', 'description', <%=!writePermission%>)" title="Edit in larger window"><base:icon image="zoom.gif" /></a>
			</td>
		</tr>
		</table>
	</t:tab>
	</t:tabcontrol>

	<p>
	<div align=center>
		<% if (writePermission)
		{
			%>
			<table align="center">
			<tr>
				<td width="50%"><base:button onclick="saveSettings();" title="Save" /></td>
				<td width="50%"><base:button onclick="window.close();" title="Cancel" /></td>
			</tr>
			</table>
			<%
		}
		else
		{
			%>
			<div class="error">You do not have permission to change this information</div>
			<table align="center">
			<tr>
				<td><base:button onclick="window.close();" title="Close" /></td>
			</tr>
			</table>
			<%
		}
		%>
	</div>
	</form>
	</base:body>
	</base:page>
	<%
}
finally
{
	if (dc != null) dc.close();
}
%>
